/*   File: AI_verify.cc
     Computes the integrals ...
     

     Author:  Tomas Johnson <johnson@math.uu.se>
     Latest edit: fre maj 22 13:43:40 CET 2008

     By: Tomas Johnson <johnson@math.uu.se>
*/

#include <fstream>
#include <cstdlib>
#include <iostream>
#include <string>
#include <stdexcept>
#include <time.h>
#include "imath.hpp"
#include "interval.hpp"
#include "ivector.hpp"
#include "imatrix.hpp"
#include "matinv.hpp"

#include "list.h"

//#include "grad_ari.hpp"
//#include "hess_ari.hpp"

using namespace cxsc;
using namespace std;

//Performance variables:
int intYes(0);   //counts the number of boxes covering the oval
int intNo(0);    //counts the number of boxes labeled fail

//Global constants, initiated at run-time:
interval lambda;
interval mu;
int oval;   //1=left, 2=right, 3=out
int d;


enum command {START_TIMING, SHOW_TIMING, STOP_TIMING};
////////////////////////////////////////////////////////////////////////////
//
// 'START_TIMING' starts the clock, 'STOP_TIMING' stops the clock,
// and prints the elapsed time in hh:mm:ss. 'SHOW_TIMING' prints the
// elapsed time.
//
void clock(const command &cmd)
{
  struct tm     *ptr;          // Used for timing the program
  static time_t  time_start;   // start time

  if ( cmd == START_TIMING ) {
    time_start = time(NULL);
    ptr = localtime(&time_start);
    cout << "Started computations: " << asctime(ptr);
  }
  else if ( cmd == SHOW_TIMING || cmd == STOP_TIMING ) {
    int weeks   = 0;
    int days    = 0;
    int hours   = 0;
    int minutes = 0;
    double seconds;
    time_t time_stop;

    time_stop = time(NULL);
    ptr = localtime(&time_stop);
    if ( cmd == STOP_TIMING )
      cout << "Completed computations: " << asctime(ptr);
    seconds = difftime(time_stop, time_start);
    cout << "Elapsed run time: ";
    // Convert seconds to vv:dd:hh:mm:ss
    while ( seconds >= 60 ) { seconds -= 60; minutes++; }
    while ( minutes >= 60 ) { minutes -= 60;   hours++; }
    while (   hours >= 24 ) {   hours -= 24;    days++; }
    while (    days >=  7 ) {    days -=  7;   weeks++; }
    if ( weeks > 0)
      cout << weeks << " week" << (weeks != 1 ? "s, " : ", ");
    if ( days > 0)
      cout << days << " day" << (days != 1 ? "s, " : ", ");
    if ( hours > 0)
      cout << hours << " hour" << (hours != 1 ? "s, " : ", ");
    if ( minutes > 0)
      cout << minutes << " minute" << (minutes != 1 ? "s, " : ", ");
    cout << seconds << " second" << (seconds != 1 ? "s." : ".") << endl;
  }
}

////////////////////////////////////////////////////////////////////////////
//if two intervals intersect this function returns true, and puts the
//intersection in res, otherwise it returns false
bool Intersect(interval &res, const interval &in1, const interval &in2) {
  if(Sup(in1)<Inf(in2) || Sup(in2)<Inf(in1))
    return false;
  res=in1&in2;
  return true;
}


//Vector version of the fcn above
/////////////////////////////////////////////////////////////////////
bool Intersect(ivector &res, const ivector &in1, const ivector &in2) {
  int n=Ub(in1);
  if(n!=Ub(in2))
    return false;
  for(int i=1; i<=n; i++) {
    if (!Intersect(res[i], in1[i], in2[i]) )
      return false;
  }
  return true;
}

//////////////////////////////////////////////////////////////////////////
//HYPER ELLIPTIC ORDER 5
/////////////////////////////////////////////////////////////////////////
//The functions below evaluate the Hyper-Elliptic Hamiltonian
//and its gradient, given the values of lambda, mu and a box
//////////////////////////////////////////////////////////////////////////

ivector gradH(const ivector &Box) {
  ivector res(2);
  res[1]=power(Box[1],4)-(1.0+lambda+mu)*power(Box[1],3)+(lambda+mu+lambda*mu)*sqr(Box[1])-lambda*mu*Box[1];
  res[2]=Box[2];
  return res;
}

rvector gradH(const rvector &onOval) {
  return mid(gradH(ivector(onOval)));
}

interval H(const ivector &Box) {
  ivector grad=gradH(Box);
  interval xprim(grad[1]);
  interval yprim(grad[2]);

  //check for monotonicity
  int xMono=in(0.0, xprim);
  int yMono=in(0.0, yprim);

  interval res=sqr(Box[2])/2.0+sqr(Box[1])*(power(Box[1],3)/5.0-(1.0+lambda+mu)*sqr(Box[1])/4.0+(lambda+mu+lambda*mu)*Box[1]/3.0-lambda*mu/2.0);

  //if monotonic in one or both coordinates, use this fact
  if (xMono==0 && yMono==0)
    res=res&( sqr(Inf(Box[2]))/2.0+sqr(Inf(Box[1]))*(power(Inf(Box[1]),3)/5.0-(1.0+lambda+mu)*sqr(Inf(Box[1]))/4.0+(lambda+mu+lambda*mu)*Inf(Box[1])/3.0-lambda*mu/2.0) |
	      sqr(Inf(Box[2]))/2.0+sqr(Sup(Box[1]))*(power(Sup(Box[1]),3)/5.0-(1.0+lambda+mu)*sqr(Sup(Box[1]))/4.0+(lambda+mu+lambda*mu)*Sup(Box[1])/3.0-lambda*mu/2.0) |
	      sqr(Sup(Box[2]))/2.0+sqr(Inf(Box[1]))*(power(Inf(Box[1]),3)/5.0-(1.0+lambda+mu)*sqr(Inf(Box[1]))/4.0+(lambda+mu+lambda*mu)*Inf(Box[1])/3.0-lambda*mu/2.0) |
	      sqr(Sup(Box[2]))/2.0+sqr(Sup(Box[1]))*(power(Sup(Box[1]),3)/5.0-(1.0+lambda+mu)*sqr(Sup(Box[1]))/4.0+(lambda+mu+lambda*mu)*Sup(Box[1])/3.0-lambda*mu/2.0) );

  else if (xMono==0)
    res=res&( sqr(Box[2])/2.0+sqr(Inf(Box[1]))*(power(Inf(Box[1]),3)/5.0-(1.0+lambda+mu)*sqr(Inf(Box[1]))/4.0+(lambda+mu+lambda*mu)*Inf(Box[1])/3.0-lambda*mu/2.0) |
	      sqr(Box[2])/2.0+sqr(Sup(Box[1]))*(power(Sup(Box[1]),3)/5.0-(1.0+lambda+mu)*sqr(Sup(Box[1]))/4.0+(lambda+mu+lambda*mu)*Sup(Box[1])/3.0-lambda*mu/2.0) );
  
  else if (yMono==0)
    res=res&( sqr(Inf(Box[2]))/2.0+sqr(Box[1])*(power(Box[1],3)/5.0-(1.0+lambda+mu)*sqr(Box[1])/4.0+(lambda+mu+lambda*mu)*Box[1]/3.0-lambda*mu/2.0) |
	      sqr(Sup(Box[2]))/2.0+sqr(Box[1])*(power(Box[1],3)/5.0-(1.0+lambda+mu)*sqr(Box[1])/4.0+(lambda+mu+lambda*mu)*Box[1]/3.0-lambda*mu/2.0) );

  interval HMid=sqr(mid(Box[2]))/2.0+sqr(mid(Box[1]))*(power(mid(Box[1]),3)/5.0-(1.0+lambda+mu)*sqr(mid(Box[1]))/4.0+(lambda+mu+lambda*mu)*mid(Box[1])/3.0-lambda*mu/2.0);

  //intersect with centred form
  res=res&(HMid+xprim*(Box[1]-mid(Box[1]))+yprim*(Box[2]-mid(Box[2])));

  return res;
}


/////////////////////////////////////////////////////////////////////////////////////////
//Integrators
/////////////////////////////////////////////////////////////////////////////////////////

//On boxes labeled inside, integrate without truncation error
void Integrate(const int &i, imatrix &Imono, const ivector Box) {
  Imono[i][1]+=interval(diam(Box[1])*diam(Box[2]));
  Imono[i][2]+=interval(sqr(Sup(Box[1]))-sqr(Inf(Box[1])))/2.0*diam(Box[2]);
  Imono[i][3]+=interval(power(Sup(Box[1]),3)-power(Inf(Box[1]),3))/3.0*diam(Box[2]);
  Imono[i][4]+=interval(power(Sup(Box[2]),3)-power(Inf(Box[2]),3))/3.0*diam(Box[1]);
  if(d==6) {
    Imono[i][5]+=interval(power(Sup(Box[1]),4)-power(Inf(Box[1]),4))/4.0*diam(Box[2]);
    Imono[i][6]+=interval(power(Sup(Box[2]),3)-power(Inf(Box[2]),3))/3.0*interval(sqr(Sup(Box[1]))-sqr(Inf(Box[1])))/2.0;
  }
  
}

//On boxes labeled fail, integrate and take hull of all possible outcomes
void boundaryIntegrate(const int &i, imatrix &Imono, const ivector &Box) {
  Imono[i][1]+=interval(0.0, diam(Box[1])*diam(Box[2]) );
  Imono[i][3]+=interval(0.0)|interval(power(Sup(Box[1]),3)-power(Inf(Box[1]),3))/3.0*diam(Box[2]);
  Imono[i][4]+=interval(0.0)|interval(power(Sup(Box[2]),3)-power(Inf(Box[2]),3))/3.0*diam(Box[1]);
  if( !in(0.0, Box[1]) && !in(0.0,Box[2]) ) {
    Imono[i][2]+=interval(0.0)|interval(sqr(Sup(Box[1]))-sqr(Inf(Box[1])))/2*diam(Box[2]);
    if(d==6) {
      Imono[i][5]+=interval(0.0)|interval(power(Sup(Box[1]),4)-power(Inf(Box[1]),4))/4.0*diam(Box[2]);
      Imono[i][6]+=interval(0.0)|interval(power(Sup(Box[2]),3)-power(Inf(Box[2]),3))/3.0*interval(sqr(Sup(Box[1]))-sqr(Inf(Box[1])))/2.0;
    }
  }
  else if( !in(0.0, Box[2]) ) {
    Imono[i][2]+=interval(0.0)|interval(sqr(Sup(Box[1]))+sqr(Inf(Box[1])))/2*diam(Box[2]);
    if(d==6) {
      Imono[i][5]+=interval(0.0)|interval(power(Sup(Box[1]),4)+power(Inf(Box[1]),4))/4.0*diam(Box[2]);
      Imono[i][6]+=interval(0.0)|interval(power(Sup(Box[2]),3)-power(Inf(Box[2]),3))/3.0*interval(sqr(Sup(Box[1]))+sqr(Inf(Box[1])))/2.0;
    }
  }
  else if( !in(0.0, Box[1]) ) {
    Imono[i][2]+=interval(0.0)|interval(sqr(Sup(Box[1]))-sqr(Inf(Box[1])))/2*diam(Box[2]);
    if(d==6) {
      Imono[i][5]+=interval(0.0)|interval(power(Sup(Box[1]),4)-power(Inf(Box[1]),4))/4.0*diam(Box[2]);
      Imono[i][6]+=interval(0.0)|interval(power(Sup(Box[2]),3)-power(Inf(Box[2]),3))/3.0*interval(sqr(Sup(Box[1]))-sqr(Inf(Box[1])))/2.0;
    }
    
  }
  else {
    Imono[i][2]+=interval(0.0)|interval(sqr(Sup(Box[1]))+sqr(Inf(Box[1])))/2*diam(Box[2]);   
    if(d==6) {
      Imono[i][5]+=interval(0.0)|interval(power(Sup(Box[1]),4)+power(Inf(Box[1]),4))/4.0*diam(Box[2]);
      Imono[i][6]+=interval(0.0)|interval(power(Sup(Box[2]),3)-power(Inf(Box[2]),3))/3.0*interval(sqr(Sup(Box[1]))+sqr(Inf(Box[1])))/2.0;
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////
//Intersection checks
////////////////////////////////////////////////////////////////////////////////////////
//check if the lines between p_up/down and q_up/down, intersect the oval
//by checking if either H|line-h is or (H|line)' is nonzero.
////////////////////////////////////////////////////////////////////////////////////////
bool intersectOval(const interval &k, ivector &left, 
		      const interval &indep, const int &indepIndex, const interval &h) {
  // cout<<indep<<endl;
  bool value(false), diff(false);
  if(indepIndex==1) {
    for(int i=0; i<=199; i++){
      ivector cand(2);
      cand[1]=Inf(indep)+interval(i, i+1.0)/200.0*interval(Sup(indep)-Inf(indep));
      //  cout<<i<<" "<<cand[1];
      cand[2]=left[2]+k*(cand[1]-left[1]);
      ivector grad=gradH(cand);
      interval Hprim=grad[1]+k*grad[2];   
      interval temp;
      if( in(0.0,Hprim) )
	diff=true;
      if(Intersect(temp, h, H(cand)) )
	value=true;
    }
  }
  else {
    for(int i=0; i<=199; i++){
      ivector cand(2);
      cand[2]=Inf(indep)+interval(i, i+1.0)/200.0*interval(Sup(indep)-Inf(indep));
      cand[1]=left[1]+k*(cand[2]-left[2]);
      ivector grad=gradH(cand);
      interval Hprim=k*grad[1]+grad[2];   
      interval temp;
      if( in(0.0,Hprim) )
	diff=true;
      if(Intersect(temp, h, H(cand)) )
	value=true;
    }
  }
  //  cout<<endl;
  return value&&diff;
}

//////////////////////////////////////////////////////////////////
//Check that the oval doesn't intersect the two other sides
//////////////////////////////////////////////////////////////////
bool ovalBoundaryIntersect(const interval &h, const ivector &Box, const int &Side) {
  ivector cand(2);
  ivector encl(2);
  int n;
  if(Side==1) {
    cand[1]=interval(Sup(Box[1]));
    encl[1]=interval(Sup(Box[1]));
    n=2;
  }
  else if(Side==2) {
    cand[2]=interval(Sup(Box[2]));
    encl[2]=interval(Sup(Box[2]));
    n=1;
  }
  else if(Side==3) {
   cand[1]=interval(Inf(Box[1]));
   encl[1]=interval(Inf(Box[1]));
   n=2;
  }
  else if(Side==4) {
    cand[2]=interval(Inf(Box[2]));
    encl[2]=interval(Inf(Box[2]));
    n=1;
  }

  cand[n]=interval(mid(Box[n]));
  encl[n]=Box[n];
  
  bool intersect(true);
  
  for(int i=1; i<=8; i++) {
    interval Hmh=H(cand)-h;
    interval Hmhprim=(gradH(encl))[n];
    if(in(0, Hmhprim))
      return true;
    
    interval N=cand[n]-Hmh/Hmhprim;
    if(!Intersect(encl[n],encl[n],N)) {
      i=10;
      intersect=false;
    }
    else
      cand[n]=interval(mid(encl[n]));
  }
 
  return intersect;
}

////////////////////////////////////////////////////////
//Find unique intersection on one side
//returns true if the intersection is unique
//returns fail if no intersection or if the uniqueness
//proof fails.
////////////////////////////////////////////////////////
bool zeroHOfY(const interval &h, 
	      const real &candX, real &candY, interval searchY){
  ivector cand(2);
  cand[1]=interval(candX);
  cand[2]=interval(candY);
  ivector encl(2);
  encl[1]=interval(candX);
  bool unique(false);
  for(int i=1; i<=5; i++) {
    encl[2]=searchY;
    interval Hmh=H(cand)-h;
    interval Hmhprim=(gradH(encl))[2];
    if(in(0, Hmhprim)) {
      candY=mid(cand[2]);
      return false;
    }
    interval N=cand[2]-Hmh/Hmhprim;
    if(in(N,searchY))
      unique=true;
    if(!Intersect(searchY,searchY,N)) {
      candY=mid(cand[2]);
      return false;
    }
    cand[2]=interval(mid(searchY));
  }
  candY=mid(cand[2]);
  return unique;
}

bool zeroHOfX(const interval &h, 
	      real &candX, const real &candY, interval searchX){
  ivector cand(2);
  cand[1]=interval(candX);
  cand[2]=interval(candY);
  ivector encl(2);
  encl[2]=interval(candY);
  bool unique(false);
  for(int i=1; i<=5; i++) {
    encl[1]=searchX;
    interval Hmh=H(cand)-h;
    interval Hmhprim=(gradH(encl))[1];
    if(in(0, Hmhprim)) {
      candX=mid(cand[1]);
      return false;
    }
    interval N=cand[1]-Hmh/Hmhprim;
    if(in(N,searchX))
      unique=true;
    if(!Intersect(searchX,searchX,N)) {
      candX=mid(cand[1]);
      return false;
    }
    cand[1]=interval(mid(searchX));
  }
  candX=mid(cand[1]);
  return unique;
}
/////////////////////////////////////////////////////////////////////////////////////
//Integrate on a triangle in 'general' position
//Zeroth order quadrature scheme, ok since the triangles are so small.
/////////////////////////////////////////////////////////////////////////////////////

bool triangle(const int &Index, const ivector &C1, const ivector &C2, const ivector &C3, 
	      const interval &h, const int &i, imatrix &Imono) {
  interval triArea;
  ivector triEnclosure;
  triEnclosure=C1|C2|C3;

  if(Index==1){
    ivector testPt=(C1+C2+C3)/3;
    interval Htest=H(testPt);
    if(Inf(Htest)>Sup(h))
      return true;
    else if(Sup(Htest)<Inf(h)) {
      interval x0=interval(Inf(C3[1]));
      interval x1=interval(Sup(C1[1]));
      interval y0=interval(Inf(C1[2]));
      interval y1=interval(Sup(C2[2]));
      if(x1==x0 || y0==y1)
	return true;

      triArea=(y1-y0)*(x1-x0)/2.0;
    }
    else {cout<<"err"<<endl;
      return false; }
    
  }
  else if(Index==2) {
    ivector testPt=(C1+C2+C3)/3;
    interval Htest=H(testPt);
    if(Inf(Htest)>Sup(h))
      return true;
    else if(Sup(Htest)<Inf(h)) {
      interval x0=interval(Inf(C2[1]));
      interval x1=interval(Sup(C1[1]));
      interval y0=interval(Inf(C3[2]));
      interval y1=interval(Sup(C1[2]));
      if(x1==x0 || y0==y1)
	return true;

      triArea=(y1-y0)*(x1-x0)/2.0;
    }
    else {cout<<"err"<<endl;
      return false;}
  }
  else if(Index==3){
    ivector testPt=(C1+C2+C3)/3;
    interval Htest=H(testPt);
    if(Inf(Htest)>Sup(h))
      return true;
    else if(Sup(Htest)<Inf(h)) {
      interval x0=interval(Inf(C2[1]));
      interval x1=interval(Sup(C1[1]));
      interval y0=interval(Inf(C3[2]));
      interval y1=interval(Sup(C2[2]));
      if(x1==x0 || y0==y1)
	return true;

      triArea=(y1-y0)*(x1-x0)/2.0;
    }
    else {cout<<"err"<<endl;
      return false;}
  }
  else{  //Index==4
    ivector testPt=(C1+C2+C3)/3;
    interval Htest=H(testPt);
    if(Inf(Htest)>Sup(h))
      return true;
    else if(Sup(Htest)<Inf(h)) {
      interval x0=interval(Inf(C3[1]));
      interval x1=interval(Sup(C1[1]));
      interval y0=interval(Inf(C3[2]));
      interval y1=interval(Sup(C2[2]));
      if(x1==x0 || y0==y1)
	return true;

      triArea=(y1-y0)*(x1-x0)/2.0;
    }
    else {cout<<"err"<<endl;
      return false;}
  }

  triArea=abs(triArea);

  Imono[i][1]+=triArea;
  Imono[i][2]+=triArea*triEnclosure[1];
  Imono[i][3]+=triArea*sqr(triEnclosure[1]);
  Imono[i][4]+=triArea*sqr(triEnclosure[2]);
  if(d==6) {
    Imono[i][5]+=triArea*power(triEnclosure[1],3);
    Imono[i][6]+=triArea*triEnclosure[1]*sqr(triEnclosure[2]);
  }

  return true;

}
/////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////
//Rotate the oval and integrate on the boxes and triangles
/////////////////////////////////////////////////////////////////////////////////////////////////////
bool boundaryIntegrateRot(const int &i, interval h, imatrix &Imono, const ivector &Box, const real &tol) {
 
  rvector onOval=mid(Box);
  rvector grad=gradH( onOval);
  if(abs(grad[1])>abs(grad[2]))
    zeroHOfX(h, onOval[1], onOval[2], Box[1]);
  else
    zeroHOfY(h,onOval[1], onOval[2], Box[2]);

  grad=gradH( onOval);              
 
  rvector appOval(2);
  appOval[1]=-grad[2];
  appOval[2]=grad[1];

  if(appOval[1]<0.0)
    appOval=-appOval;
  real rightX, leftX, rightY, leftY;
  int right, left;

  real gradTol=power(10,-14);
  if(appOval[2]>0.0) {
    if(abs(appOval[2])< gradTol) {
      rightX=Sup(Box[1]);
      leftX=Inf(Box[1]);
      right=1;
      left=3;
      rightY=onOval[2];
      leftY=onOval[2];
    } 
    else if(abs(appOval[1])< gradTol) {
      rightX=onOval[1];
      leftX=onOval[1];
      right=2;
      left=4;
      rightY=Sup(Box[2]);
      leftY=Inf(Box[2]);      
    }
    else {
      rightX=min(Sup(Box[1]), onOval[1]+(Sup(Box[2])-onOval[2])*appOval[1]/appOval[2]);
      if(rightX==Sup(Box[1]))
	right=1;
      else
	right=2;
      leftX =max(Inf(Box[1]), onOval[1]+(Inf(Box[2])-onOval[2])*appOval[1]/appOval[2]);
    
      if(leftX==Inf(Box[1]))
	left=3;
      else
	left=4;
      
      rightY=min(Sup(Box[2]), onOval[2]+(Sup(Box[1])-onOval[1])*appOval[2]/appOval[1]);
      leftY =max(Inf(Box[2]), onOval[2]+(Inf(Box[1])-onOval[1])*appOval[2]/appOval[1]);
    }
  }  
  else {
    if(abs(appOval[2])< gradTol) {
      rightX=Sup(Box[1]);
      leftX=Inf(Box[1]);
      right=1;
      left=3;
      rightY=onOval[2];
      leftY=onOval[2];
    }
    else if(abs(appOval[1])< gradTol) {
      rightX=onOval[1];
      leftX=onOval[1];
      right=4;
      left=2;
      rightY=Inf(Box[2]);
      leftY=Sup(Box[2]);     
    }
    else {
      rightX=min(Sup(Box[1]), onOval[1]+(Inf(Box[2])-onOval[2])*appOval[1]/appOval[2]);
      if(rightX==Sup(Box[1]))
	right=1;
      else
	right=4;
      leftX =max(Inf(Box[1]), onOval[1]+(Sup(Box[2])-onOval[2])*appOval[1]/appOval[2]);
      if(leftX==Inf(Box[1]))
	left=3;
      else
	left=2;
   
      rightY=max(Inf(Box[2]), onOval[2]+(Sup(Box[1])-onOval[1])*appOval[2]/appOval[1]);
      leftY =min(Sup(Box[2]), onOval[2]+(Inf(Box[1])-onOval[1])*appOval[2]/appOval[1]);
    }
  }
  
  //Now locate a zero of H-h on the intersecting sides !
  //Then we
  //Must prove that the lines from rightUp to leftUp and viceversa 
  // are outside of the spectrum !
  ivector rightUp(2), leftUp(2), rightDown(2), leftDown(2);
  if (right==1) {
    if(!zeroHOfY(h, rightX, rightY, Box[2]))
      return false;
    rightUp[1]=interval(Sup(Box[1]));
    rightUp[2]=interval(min(rightY+tol, Sup(Box[2])));
    rightDown[1]=interval(Sup(Box[1]));
    rightDown[2]=interval(max(rightY-tol, Inf(Box[2]))); 
  }
  else if(right==2) {
    if(!zeroHOfX(h, rightX, rightY, Box[1]))
      return false;
    rightUp[1]=interval(max(rightX-tol, Inf(Box[1])));
    rightUp[2]=interval(Sup(Box[2]));
    rightDown[1]=interval(min(rightX+tol,Sup(Box[1])));
    rightDown[2]=interval(Sup(Box[2]));      
  }
  else { //4
    if(!zeroHOfX(h, rightX, rightY, Box[1]))
      return false;
    rightUp[1]=interval(min(rightX+tol, Sup(Box[1])));
    rightUp[2]=interval(Inf(Box[2]));
    rightDown[1]=interval(max(rightX-tol, Inf(Box[1])));
    rightDown[2]=interval(Inf(Box[2]));   
  }
  
  if (left==3) {
    if(!zeroHOfY(h, leftX, leftY, Box[2]))
      return false;
    leftUp[1]=interval(Inf(Box[1]));
    leftUp[2]=interval(min(leftY+tol, Sup(Box[2])));
    leftDown[1]=interval(Inf(Box[1]));
    leftDown[2]=interval(max(leftY-tol, Inf(Box[2])));      
  }
  else if(left==2) {
    if(!zeroHOfX(h, leftX, leftY, Box[1]))
      return false;
    leftUp[1]=interval(min(leftX+tol, Sup(Box[1])));
    leftUp[2]=interval(Sup(Box[2]));
    leftDown[1]=interval(max(leftX-tol, Inf(Box[1])));
    leftDown[2]=interval(Sup(Box[2]));      
  }
  else { //4
    if(!zeroHOfX(h, leftX, leftY, Box[1]))
      return false;
    leftUp[1]=interval(max(leftX-tol, Inf(Box[1])));
    leftUp[2]=interval(Inf(Box[2]));
    leftDown[1]=interval(min(leftX+tol, Sup(Box[1])));
    leftDown[2]=interval(Inf(Box[2]));      
  }

  int notSide1, notSide2;
  if(right==1 && left==2) {
    notSide1=3;
    notSide2=4;
  }
  else if(right==1 && left==3) {
    notSide1=2;
    notSide2=4;
  }
  else if(right==1 && left==4) {
    notSide1=2;
    notSide2=3;
  }
  else if(right==2 && left==3) {
    notSide1=1;
    notSide2=4;
  }
  else if((right==2 && left==4) || (right==4 && left==2) ){
    notSide1=1;
    notSide2=3;
  }
  else if(right==4 && left==3) {
    notSide1=1;
    notSide2=2;
  }

  if(ovalBoundaryIntersect( h, Box, notSide1) ||
     ovalBoundaryIntersect( h, Box, notSide2) ) {
    return false;
  }

  //Make sure that up and down are on different sides
  //Suffices to prove for one if the test below works.
  if( !(Sup((H( rightDown)-h)*(H( rightUp)-h)) < 0.0) )
    return false;

  //Make sure both up and down are on the same side
  if ( !( 0.0 < Inf((H( rightUp)-h)*(H(leftUp)-h)) 
	  &&  0.0 < Inf((H( rightDown)-h)*(H(leftDown)-h)) ) )
    return false;
 
  if(abs(grad[2])>abs(grad[1])) {
    if(in(0.0,rightUp[1]-leftUp[1]))
      return false;
   
    interval k=(rightUp[2]-leftUp[2])/(rightUp[1]-leftUp[1]);
    interval x=leftUp[1]|rightUp[1];
    
    if( intersectOval( k, leftUp,x,1,h) )
      return false;

    if(in(0.0,rightDown[1]-leftDown[1])) 
      return false;
   
    k=(rightDown[2]-leftDown[2])/(rightDown[1]-leftDown[1]);
    x=leftDown[1]|rightDown[1];
 
    if( intersectOval( k, leftDown,x,1,h) )
      return false; 
 
  }

  else {
    if(in(0.0,rightUp[2]-leftUp[2]))
      return false;
    
    interval k=(rightUp[1]-leftUp[1])/(rightUp[2]-leftUp[2]);
    interval y=leftUp[2]|rightUp[2];

    if( intersectOval( k, leftUp,y,2,h) )
      return false; 
    
    
    if(in(0.0,rightDown[2]-leftDown[2]))
      return false;
    
    k=(rightDown[1]-leftDown[1])/(rightDown[2]-leftDown[2]);
    y=leftDown[2]|rightDown[2];
  
    if( intersectOval( k, leftDown,y,2,h) )
      return false;
  
  }
  
  //Time to integrate !!!
    
  interval errArea;
  ivector Box1(2), Box2(2);
  if (right==1 && left==2) {
    errArea=((leftDown[1]-Sup(Box[1]))*(rightDown[2]-Sup(Box[2]))-(leftUp[1]-Sup(Box[1]))*(rightUp[2]-Sup(Box[2])))/2;
    if(!triangle(1, rightUp, ivector(Sup(Box)), leftUp,  h, i, Imono))
      return false;
    ivector temp(2);
    temp[1]=Inf(leftDown[1]);
    temp[2]=Inf(rightDown[2]);
    if(!triangle(3, rightDown, leftDown, temp,  h, i, Imono))
      return false;
 
    Box1[1]=interval(Inf(Box[1]), Inf(leftDown[1]));
    Box1[2]=Box[2];
    Box2[1]=interval(Inf(leftDown[1]), Sup(Box[1]));
    Box2[2]=interval(Inf(Box[2]), Inf(rightDown[2]));

  }
  else if (right==1 && left==3) {
    errArea=2*tol*diam(Box[1]); 

    if(Inf(rightUp[2])>Sup(leftUp[2])) {
      ivector temp(2);
      temp[1]=Inf(Box[1]);
      temp[2]=Sup(rightUp[2]);
      if(!triangle(2, rightUp, temp, leftUp,  h, i, Imono))
	return false;
    
      temp[1]=Sup(Box[1]);
      temp[2]=Inf(leftDown[2]);
      if(!triangle(4, temp, rightDown, leftDown,  h, i, Imono))
	return false;
    }
    else {
      ivector temp(2);
      temp[1]=Sup(Box[1]);
      temp[2]=Sup(leftUp[2]);
      if(!triangle(1, rightUp, temp, leftUp,  h, i, Imono))
	return false;

      temp[1]=Inf(Box[1]);
      temp[2]=Inf(rightDown[2]);
      if(!triangle(3, rightDown, leftDown, temp,  h, i, Imono))
	return false;
    }

    Box1[1]=Box[1];
    Box1[2]=interval(max(Sup(rightUp[2]),Sup(leftUp[2])), Sup(Box[2]));
    Box2[1]=Box[1];
    Box2[2]=interval(Inf(Box[2]), min(Inf(rightDown[2]), Inf(leftDown[2])));
  }
  else if (right==1 && left==4) {
    errArea=((leftUp[1]-Sup(Box[1]))*(rightUp[2]-Inf(Box[2]))-(leftDown[1]-Sup(Box[1]))*(rightDown[2]-Inf(Box[2])))/2;

    ivector temp(2);
    temp[1]=Inf(leftUp[1]);
    temp[2]=Sup(rightUp[2]);
    if(!triangle(2, rightUp, temp, leftUp,  h, i, Imono))
      return false;
    
    temp[1]=Sup(Box[1]);
    temp[2]=Inf(Box[2]);
    if(!triangle(4, temp, rightDown, leftDown,  h, i, Imono))
      return false;

    Box1[1]=interval(Inf(Box[1]), Inf(leftUp[1]));
    Box1[2]=interval(Inf(Box[2]), Sup(rightUp[2]));
    Box2[1]=Box[1];
    Box2[2]=interval(Sup(rightUp[2]), Sup(Box[2]));

  }
  else if(right==2 && left==3) {
    errArea=((leftDown[2]-Sup(Box[2]))*(rightDown[1]-Inf(Box[1]))-(leftUp[2]-Sup(Box[2]))*(rightUp[1]-Inf(Box[1])))/2; 
    ivector temp(2);
    temp[1]=Inf(Box[1]);
    temp[2]=Sup(Box[2]);
    if(!triangle(2, rightUp, temp, leftUp,  h, i, Imono))
      return false;
    
    temp[1]=Sup(rightDown[1]);
    temp[2]=Inf(leftDown[2]);
    if(!triangle(4, temp, rightDown, leftDown,  h, i, Imono))
      return false;
    
    Box1[1]=interval(Sup(rightDown[1]), Sup(Box[1]));
    Box1[2]=interval(Sup(leftUp[2]), Sup(Box[2]));
    Box2[1]=Box[1];
    Box2[2]=interval(Inf(Box[2]), Sup(leftUp[2]));

  }
  else if((right==2 && left==4) || (right==4 && left==2) ) {
    errArea=2*tol*diam(Box[2]);   

    if(right==2) {
      ivector temp(2);
      temp[1]=Inf(leftUp[1]);
      temp[2]=Sup(Box[2]);
      if(!triangle(2, rightUp, temp, leftUp,  h, i, Imono))
	return false;
      
      temp[1]=Sup(rightDown[1]);
      temp[2]=Inf(Box[2]);
      if(!triangle(4, temp, rightDown, leftDown,  h, i, Imono))
	return false;
    }
    else {
      ivector temp(2);
      temp[1]=Sup(rightUp[1]);
      temp[2]=Sup(Box[2]);
      if(!triangle(1, rightUp, temp, leftUp,  h, i, Imono))
	return false;
      
      temp[1]=Inf(leftDown[1]);
      temp[2]=Inf(Box[2]);
      if(!triangle(3, rightDown, leftDown, temp,  h, i, Imono))
	return false;
    }

    Box1[1]=interval(Inf(Box[1]), min(Inf(leftDown[1]), Inf(leftUp[1])));
    Box1[2]=Box[2];
    Box2[1]=interval(max(Sup(rightDown[1]), Sup(rightUp[1])), Sup(Box[1]));
    Box2[2]=Box[2];
    
  }
  else if(right==4 && left==3) {
    errArea=((leftUp[2]-Inf(Box[2]))*(rightUp[1]-Inf(Box[1]))-(leftDown[2]-Inf(Box[2]))*(rightDown[1]-Inf(Box[1])))/2;
    ivector temp(2);
    temp[1]=Sup(rightUp[1]);
    temp[2]=Sup(leftUp[2]);
    if(!triangle(1, rightUp, temp, leftUp,  h, i, Imono))
      return false;
    
    temp=Inf(Box);
    if(!triangle(3, rightDown, leftDown, temp,  h, i, Imono))
      return false;
    Box1[1]=interval(Sup(rightUp[1]), Sup(Box[1]));
    Box1[2]=interval(Inf(Box[2]), Sup(leftUp[2]));
    Box2[1]=Box[1];
    Box2[2]=interval(Sup(leftUp[2]), Sup(Box[2]));
  }

  interval htest=H(ivector(mid(Box1)));
  if(Inf(htest)>Sup(h)) {}
  else if(Sup(htest)<Inf(h))
    Integrate(i,  Imono, Box1);
  else
    return false;

  htest=H(ivector(mid(Box2)));
  if(Inf(htest)>Sup(h)) {}
  else if(Sup(htest)<Inf(h))
    Integrate(i,  Imono, Box2);
  else
    return false;
  
  ivector errBox=leftDown|leftUp|rightUp|rightDown;
  //  cout<<errArea<<endl;
  errArea=abs(errArea);

  Imono[i][1]+=interval(0.0)|errArea;
  Imono[i][2]+=interval(0.0)|errArea*errBox[1];
  Imono[i][3]+=interval(0.0)|errArea*sqr(errBox[1]);
  Imono[i][4]+=interval(0.0)|errArea*sqr(errBox[2]);
  if(d==6) {
    Imono[i][5]+=interval(0.0)|errArea*power(errBox[1],3);
    Imono[i][6]+=interval(0.0)|errArea*errBox[1]*sqr(errBox[2]);
  }

  return true;
}
/////////////////////////////////////////////////////////////////////////////////////////////////////

//Split a box in 4
void splitAndStore(const ivector &Box, List<ivector> *cover) {
  ivector temp(2);
  temp[1]=interval(Inf(Box[1]),mid(Box[1]));
  temp[2]=interval(Inf(Box[2]),mid(Box[2]));
  *cover+=temp;
  temp[2]=interval(mid(Box[2]),Sup(Box[2]));
  *cover+=temp;
  temp[1]=interval(mid(Box[1]),Sup(Box[1]));
  *cover+=temp;
  temp[2]=interval(Inf(Box[2]),mid(Box[2]));
  *cover+=temp;
}

//Calculate I(h)
interval I(const int &Index, const interval &h, const imatrix &Imono, const ivector &coef) {
  ivector currentImono=Imono[Index];
  interval retI(0.0);

  for(int j=1; j<=d; j++) {
    retI+=coef[j]*currentImono[j];
  }

  return retI;
}

int main(int argc, char * argv[]) {
  if ( argc != 5 ) {
    cout << "Syntax: " << argv[0] << " lambda mu tol1 oval" << endl;
    exit(0);
  }
  cout << endl
       << "**********************************************************" << endl
       << "******************* Imono - Integrator ********************" << endl
       << endl << "Latest compilation: " << __DATE__ << ", " << __TIME__
       << endl << endl;
  clock(START_TIMING);
  
  cout<< SetPrecision(7,6);
  //cout<< SetPrecision(17,16);

  lambda=interval(atof(argv[1]));
  mu=interval(atof(argv[2]));
  if(Inf(mu) > Sup(lambda) ) {
    cout<<"Put lambda > mu "<<endl;
    exit(0);
  }
    
  oval=atoi(argv[4]);
  
  if (oval>3 || oval<1) {
    cout<<"Put oval = 1, oval = 2, or oval =3 "<<endl;
    exit(0);
  }

  d=6;
  
  real tol1(atof(argv[3]));
  real tol2(tol1/10.0);
  rvector hlevel(4);
  
  hlevel[1]=0.0;
  hlevel[2]=mid(-power(mu,3)/60.0*(3*sqr(mu)-5*lambda*mu-5*mu+10*lambda));
  hlevel[3]=mid(-power(lambda,3)/60.0*(3*sqr(lambda)-5*lambda*mu-5*lambda+10*mu));
  hlevel[4]=mid(-1.0/60.0*(3-5*lambda-5*mu+10*lambda*mu));
  
  bool phase;
 
  if(Inf(mu) > Sup(lambda*(3*lambda-5.0)/(5.0*(lambda-2.0))) )
    phase=0;
  else
    phase=1;

  ivector coef(d);
  /////////////////////////////////////////////
  //Different cases:
  /////////////////////////////////////////////

///////////////////////////////////////////  
//   //l=0.7, m=0.35
//////////////////////////////////////////

//   Lienard, (2,0,2)  
//   coef[1]=-0.493218693940730;
//   coef[2]=2.28749888822311;
//   coef[3]=-2.814936399538578;
//   coef[4]=0.0;
//   coef[5]=1.0;
//   coef[6]=0.0;

//   //left
//   int sampleNr(3);
//   rvector h(sampleNr);       
//   h[1]=-0.002489;
//   h[2]=-0.001578;
//   h[3]=-0.001404;

//   //out
//   int sampleNr(3);
//   rvector h(sampleNr);       
//   h[1]=-0.001398;
//   h[2]=-0.001216;
//   h[3]=-0.0008404;
////////////////////OK

//////////////////////////

// //   Lienard, (2,1,1)
//   coef[1]=-0.527151835377033;
//   coef[2]=2.412743372700620;
//   coef[3]=-2.902506452438232;
//   coef[4]=0.0;
//   coef[5]=1.0;
//   coef[6]=0.0;

//   //left
//   int sampleNr(3);
//   rvector h(sampleNr);       
//   h[1]=-0.002583;
//   h[2]=-0.001609;
//   h[3]=-0.001404;

  //right
//   int sampleNr(2);
//   rvector h(sampleNr);         
//   h[1]=-0.002251;
//   h[2]=-0.001404;

//   //out
//   int sampleNr(2);
//   rvector h(sampleNr);         
//   h[1]=-0.001168;
//   h[2]=-0.0004; 
/////////////////////OK

///////////////////////////

//   Lienard, (1,1,2)       
//  coef[1]=-0.335790039585681;
//   coef[2]=1.666698134140512;
//   coef[3]=-2.334707342050355;
//   coef[4]=0.0;
//   coef[5]=1.0;
//   coef[6]=0.0;

//   //left
//   int sampleNr(2);
//   rvector h(sampleNr);  
//   h[1]=-0.002617;
//   h[2]=-0.001435;

//   //right
//   int sampleNr(2);
//   rvector h(sampleNr);  
//   h[1]=-0.002444;
//   h[2]=-0.001439;

//   //out
 //  int sampleNr(3);
//   rvector h(sampleNr);  
//   h[1]=-0.001398;
//   h[2]=-0.001305;
//   h[3]=-0.0008404;
//////////////////OK

//////////////////////////
  // Lienard, (4,0,0)       
//   coef[1]=0.090955087198520;
//   coef[2]=-0.146673784978146;
//   coef[3]=-0.673237332307929;
//   coef[4]=0.0;
//   coef[5]=1.0;
//   coef[6]=0.0;

//   //left
//   int sampleNr(5);
//   rvector h(sampleNr);  
//   h[1]=-0.001401;
//   h[2]=-0.001440;
//   h[3]=-0.001655;
//   h[4]=-0.002168;
//   h[5]=-0.002845;
/////////////////////OK

///////////////////////////

//(2,2,2)
//   coef[1]= 0.015240986874536;
//   coef[2]=-0.081034773577249;
//   coef[3]= 0.130830332555061;
//   coef[4]=-1.131442491266112;
//   coef[5]=-0.065018719852177;
//   coef[6]= 1.000000000000000;

// //   //left
// //   int sampleNr(3);
// //   rvector h(sampleNr);  
// //   h[1]=-0.002550;
// //   h[2]=-0.001570;
// //   h[3]=-0.001401;
  
// // //   //right
// //   int sampleNr(3); 
// //   rvector h(sampleNr);  
// //   h[1]=-0.002640;
// //   h[2]=-0.001640;
// //   h[3]=-0.001401;

// //   //out
//   int sampleNr(5);
//   rvector h(sampleNr);  
//   h[1]=-0.001300;
//   h[2]=-0.0008000;
//   h[3]=-0.0005500;
//   h[4]=-0.0002000;
//   h[5]=-0.0000500;
//////////////////////OK


//(3,2,1)
//   coef[1]=  0.012579001706296;
//   coef[2]= -0.063426140322458;
//   coef[3]=  0.093641653584006;
//   coef[4]= -0.744102555169092;
//   coef[5]= -0.042925643387235;
//   coef[6]= 1.000000000000000;

// //   //left
//   int sampleNr(4);
//   rvector h(sampleNr);  
//   h[1]=-0.002856;
//   h[2]=-0.002147;
//   h[3]=-0.001658;
//   h[4]=-0.001401;
  
// // //   //right
//   int sampleNr(3); 
//   rvector h(sampleNr);  
//   h[1]=-0.002646;
//   h[2]=-0.001580;
//   h[3]=-0.001401;

// //   //out
//   int sampleNr(2);
//   rvector h(sampleNr);  
//   h[1]=-0.001314;
//   h[2]=-0.0001;
//////////////////////OK

//////////////////////////
//   (1,1,4)       

//   coef[1]= 0.000487412601086;
//   coef[2]=-0.000746501625707;
//   coef[3]= 0.000121099674506;
//   coef[4]=-0.632041452259285;
//   coef[5]=-0.000197747463660;
//   coef[6]= 1.000000000000000;


// //   //left
//   int sampleNr(2);
//   rvector h(sampleNr);
//   h[1]=-0.002288;
//   h[2]=-0.001401;

//   //right
//   int sampleNr(2);
//   rvector h(sampleNr);
//   h[1]=-0.002304;
//   h[2]=-0.001401;

//   //out
//   int sampleNr(5);
//   rvector h(sampleNr);  
//   h[1]=-0.001399;
//   h[2]=-0.001137;
//   h[3]=-0.0006478;
//   h[4]=-0.0001427;
//   h[5]=-0.000001;
////////////////////////OK

/////////////////////////////////

//   //(1,3,3)
//   coef[1]= 0.014509471576202;
//   coef[2]=-0.076198084267168;
//   coef[3]= 0.120790364287194;
//   coef[4]=-1.044988054862947;
//   coef[5]=-0.059110998792988;
//   coef[6]= 1.000000000000000;

// // //   //left
// //   int sampleNr(2);
// //   rvector h(sampleNr);  
// //   h[1]=-0.002555;
// //   h[2]=-0.001470;
  
// // //   //right
//   int sampleNr(4); 
//   rvector h(sampleNr);    
//   h[1]=-0.002845;
//   h[2]=-0.001850;
//   h[3]=-0.001495;
//   h[4]=-0.001401;
  
// // //   //out
//   int sampleNr(4);
//   rvector h(sampleNr);  
//   h[1]=-0.001399;
//   h[2]=-0.001340;
//   h[3]=-0.001146;
//   h[4]=-0.0009800;
/////////////////////OK

///////////////////////////////////////////////////

///////////////////////////

//   //(2,3,2)
 //  coef[1]= 0.014217512025472;
//   coef[2]=-0.074583260821912;
//   coef[3]= 0.118228516889348;
//   coef[4]=-1.047128604358265;
//   coef[5]=-0.057870420877864;
//   coef[6]= 1.0;

// // // //   //left
// //   int sampleNr(3);
// //   rvector h(sampleNr);  
// //   h[1]=-0.002510;
// //   h[2]=-0.001444;
// //   h[3]=-0.001401;

// //  //right
//   int sampleNr(4); 
//   rvector h(sampleNr);    
//   h[1]=-0.002900;
//   h[2]=-0.001950;
//   h[3]=-0.001497;
//   h[4]=-0.001401;

//   //out
//   int sampleNr(3);
//   rvector h(sampleNr);  
//   h[1]=-0.001360;
//   h[2]=-0.001050;
//   h[3]=-0.0008000;
//////////////////////OK


///////////////////////////////////////////////////

///////////////////////////

//   //(1,2,3)
//   coef[1]= 0.015854925218063;
//   coef[2]=-0.084647664046950;
//   coef[3]= 0.137110188186264;
//   coef[4]=-1.151207558737090;
//   coef[5]=-0.068292351789227;
//   coef[6]= 1.0;

// // //   //left
// // //   int sampleNr(2);
// // //   rvector h(sampleNr);  
// // //   h[1]=-0.002675;
// // //   h[2]=-0.001571;
  
// // //   //right
// // //   int sampleNr(3); 
// // //   rvector h(sampleNr);    
// // //   h[1]=-0.002630;
// // //   h[2]=-0.001620;
// // //   h[3]=-0.001401;
  
// // //   //out
//   int sampleNr(6);
//   rvector h(sampleNr);  
//   h[1]=-0.001398;
//   h[2]=-0.001283;
//   h[3]=-0.0008000;
//   h[4]=-0.0005000;
//   h[5]=-0.0003000;
//   h[6]=-0.0000500;
////////////////////////OK
///////////////////////////////////////////////////

//   //(2,1,3)
  coef[1]= 0.016876767465088;
  coef[2]=-0.090336905956256;
  coef[3]= 0.147748560871734;
  coef[4]=-1.304890989044986;
  coef[5]=-0.074179246592458;
  coef[6]= 1.0;

// //   //left
//   int sampleNr(3);
//   rvector h(sampleNr);  
//   h[1]=-0.002521;
//   h[2]=-0.001515;
//   h[3]=-0.001401;
  
//   //right
//   int sampleNr(2); 
//   rvector h(sampleNr);    
//   h[1]=-0.002627;
//   h[2]=-0.001482;
  
  //out
   int sampleNr(4);
   rvector h(sampleNr);  
   h[1]=-0.001399;
   h[2]=-0.001340;
   h[3]=-0.0004309;
   h[4]=-0.000003;
   ///////////////OK

///////////////////////////////////////////////////

//   //(3,1,2)
//   coef[1]= 0.013396623733006;
//   coef[2]=-0.068949475891139;
//   coef[3]= 0.105227606694727;
//   coef[4]=-0.830973679932326;
//   coef[5]=-0.049778620099614;
//   coef[6]= 1.0;

// //   //left
//   int sampleNr(4);
//   rvector h(sampleNr);  
//   h[1]=-0.002809;
//   h[2]=-0.001975;
//   h[3]=-0.001496;
//   h[4]=-0.001401;

//   //right
//   int sampleNr(2); 
//   rvector h(sampleNr);    
//   h[1]=-0.002659;
//   h[2]=-0.001507;
  
  //out
//   int sampleNr(3);
//   rvector h(sampleNr);  
//   h[1]=-0.001399;
//   h[2]=-0.001345;
//   h[3]=-0.001;
//////////////OK
///////////////////////////////////////////////////

//   //(4,0,2)
//   coef[1]=-0.003467719369657;
//   coef[2]= 0.016936137126896;
//   coef[3]=-0.022925125136071;
//   coef[4]=-0.241782262869751;
//   coef[5]= 0.008117967534894;
//   coef[6]= 1.0;

// //   //left
//   int sampleNr(5);
//   rvector h(sampleNr);  
//   h[1]=-0.002946;
//   h[2]=-0.002238;
//   h[3]=-0.001689;
//   h[4]=-0.001463;
//   h[5]=-0.001401;
  
 //  //out
//   int sampleNr(3);
//   rvector h(sampleNr);  
//   h[1]=-0.0001;
//   h[2]=-0.00002798;
//   h[3]=-0.000001;
/////////////////////OK

//////////////////////////////////////////////////
/////////////l=0.85,  m=0.35

//   //(4,2)
//   coef[1]= 0.000130024287196;
//   coef[2]= 0.001607204748556 ;
//   coef[3]=-0.007725266654496 ;
//   coef[4]=-0.331759533214819 ;
//   coef[5]= 0.005964837139651; ;
//   coef[6]= 1.0;

//   //left
//   int sampleNr(5);
//   rvector h(sampleNr);  
//   h[1]=-0.003420;
//   h[2]=-0.001696;
//   h[3]=-0.0005426;
//   h[4]=-0.0001045;
//   h[5]=-0.000001;

//   //right
//   int sampleNr(3);
//   rvector h(sampleNr);  
  //  h[1]=0.0004829;
  //  h[2]=0.0006720;
  //  h[3]=0.0007614;
  ////////////////////OK  
///////////////////////////////////////////////////
  cout<<"The coefficients of the 1-form are: "<<endl;
  cout<<coef<<endl;
  imatrix Imono(sampleNr,d); 
   
  ////////////////////////////
  for(int i=1; i<=sampleNr; i++) {
    for(int j=1; j<=6; j++) {
      Imono[i][j]=0;
    }
    
    List<ivector> cover;
    ivector Box(2);
   
    if(oval==1) {
      Box[1]=interval(0.0, Sup(lambda));
      Box[2]=interval(-1.0, 1.0);
    }
    else if (oval==2) {
      Box[1]=interval(Inf(lambda), 2.0);
      Box[2]=interval(-1.0, 1.0);
    }
    else{
      Box[1]=interval(0.0, 2.0);
      Box[2]=interval(-1.0, 1.0);
    }
    cover+=Box;
 
    while(!IsEmpty(cover)) {
      Box=Pop(cover);
      interval Htest=H( Box);
      
      if( Sup(Htest)<h[i] ) {
	Integrate(i,  Imono, Box);
      }
      else if(Inf(Htest)<=h[i] ){
	bool rotate=false;
	int p(0);
	real tempTol=tol2;
	ivector currImono=Imono[i];
	ivector orgImono=Imono[i];

	while(boundaryIntegrateRot(i,  interval(h[i]), Imono, Box, tempTol) && p<10) {
	  rotate=true;
	  p++;
	  tempTol/=2.0;
	  currImono=Imono[i];
	  Imono[i]=orgImono;
	}	  

	if(!rotate) {
	  if(max(diam(Box[1]),diam(Box[2]))<tol1) {
	    boundaryIntegrate(i, Imono, Box);
	    intNo++;
	  }      
	  else
	    splitAndStore(Box, &cover);
	}
	else {
	  Imono[i]=currImono;
	  intYes++;
	}
      }
    }

    cout<<"Completed computations of Imono for Oval number "<<i<<endl;
    cout<<"At the oval H = "<<h[i]<<endl
	<<"I(h) = "<<I(i, interval(h[i]), Imono, coef)<<endl
	<<"Midvalue Imid(h) = "<<I(i, interval(h[i]), imatrix(mid(Imono)), coef)<<endl;
  }

  cout<<endl<<"The number of boxes covering the oval is "<<intYes<<endl
      <<"The number of unidentifiable boxes is "<<intNo<<endl;
  clock(STOP_TIMING);
  cout<<Imono<<endl;
}
